module cpu(cycle, pc, inst, alu_out, mem_out, clock,
	regdst, aluop, alusrc, branch, memread, 
	memwrite, regwrite, memtoreg, zero);

	// input/output
	input clock;		// SET CLOCK INPUT IN YOUR WAVEFORM FILE
	output[31:0] cycle, pc, inst, alu_out, mem_out;
	output regdst, alusrc, branch, memread, memwrite, regwrite, memtoreg;
	output[1:0] aluop;
	output zero;
	
	// for debug
	reg[31:0] cycle;
	always @ (posedge clock) cycle = cycle + 1;

	/*
	 * "global" variables
	 */
	
	// control variables
	wire regdst, alusrc, branch, memread, memwrite, regwrite, memtoreg;
	
	// register address and data
	wire[4:0] reg_a, reg_b, reg_c /* synthesis preserve */;
	wire[31:0] val_a, val_b, val_c /* synthesis preserve */;
	
	// ALU-related variables
	wire[1:0] aluop;
	wire[2:0] alu_con;
	wire zero;

	// other variables
	reg[31:0] pc = 32'b0;
	wire[31:0] branch_target;
	wire[31:0] imm_value;

	/*
	 * inst fetch 
	 */

	// update pc
	wire[31:0] pc_plus_4 = pc + 4;
	wire branch_taken = branch & zero;
	wire[31:0] next_pc = branch_taken ? branch_target : pc_plus_4;
	always @ (posedge clock) begin
		 pc = next_pc;
	end
	
	// inst memory
	// YOUR CODE: read inst_mem.v

	/*
	 * Decode, register file (reg read and writeback)
	 */

	// Decode control signals
	wire[31:0] rom_out;
	mc_rom M1 (inst[31:26], rom_out);
	assign regdst = rom_out[8];
	// YOUR CODE: read mc_rom.mif for the positions of other control
	// signals

	// ALU control
	alu_control M2 (aluop, inst[5:0], alu_con);

	// get register numbers
	// YOUR CODE HERE: check MIPS instruction format

	// get immediate value
	// YOUR CODE: check MIPS instruction format

	// Register file: read and writeback
	// YOUR CODE: read regfile.v

	// main ALU 
	// YOUR CODE: read alu.v
	
	// branch target adder
	// YOUR CODE: check MIPS instruction format

	/*
	 * Memory read/write
	 */

	// use alu_out as memory address
	data_mem DM (alu_out[31:2], memwrite, ~clock, val_b, mem_out);
	assign val_c = memtoreg ? mem_out : alu_out;

endmodule
